#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "utils.h"
#include "structs.h"

extern struct input_options options;
char **seqs = NULL;
char **names = NULL;
char **strucs = NULL;

int readAln(void);
int read_align_only(void);
/* int read_clustal(FILE *clust, char *a_file); */
int read_fasta(FILE *fasta, char *a_file);
void free_aln_arrays();
void file_error(char *file);
void fasta_error(void);
void severe_error(int a);

/*--------------------------------------------------------------------------*/

int readAln(void) {

  int num_seq = 0, k, ch;
  FILE *a_handle = NULL, *out_handle = NULL; /* input alignment */
  char *line = NULL;

  fprintf(stdout, "\nReading input alignment ..\n");
  
  seqs = (char**)space(1000*sizeof(char*));  
  names = (char**)space(1000*sizeof(char*));
  /* seqs[0] = NULL; */
  /*   names[0] = NULL; */


    for(k=0; k<1000; k++){
         seqs[k] = NULL;
         names[k] = NULL;
       }
  
  a_handle = fopen(options.input_filename, "r");
  out_handle = fopen("tmp.tmp", "w");
  if(a_handle == NULL) {
    file_error(options.input_filename);
  }
  while((ch = fgetc (a_handle)) != EOF)	if(ch != '\015' && ch != '\032')   putc(ch, out_handle); 
  fclose(a_handle);
  fclose(out_handle);
  a_handle = fopen("tmp.tmp", "r");
  
  line=get_line(a_handle);
  if(strncmp(line, "CLUSTAL", 7) == 0) {
    fprintf(stderr, "\nThis test version can currently handle alignment input only in FASTA format.\nCLUSTALW support will be included soon. Sorry!\n\n");
    exit(11);
    
    /* num_seq = read_clustal(a_handle, options.input_filename); */
    /* if(num_seq == 0) { */
    /*     file_error(options.input_filename); */
    /*   } */
    /*     fprintf(stdout, "Done.\n"); */
    /*     return num_seq; */
  }
  
  else {
    if(line[0]== '>'){
	free(line);
      num_seq = read_fasta(a_handle, options.input_filename);
      
      if(num_seq == 0) {
	file_error(options.input_filename);
      }
      
      fprintf(stdout, ".. done.\n");
      return num_seq;
    }
    else {
      fprintf(stderr, "\nWrong format of file '%s'.\nCLUSTALW and FASTA formats are supported.\n\n", options.input_filename);
      exit(13);
    }
  }
}

/*--------------------------------------------------------------------------*/

int read_fasta(FILE *fasta, char *a_file) {

  char *line = NULL, *tmp_name = NULL, *seqs_fragment = NULL;
  int n=-1, i, j, b;

  rewind(fasta);
  line = get_line(fasta);
  if(strncmp(line,">",1) != 0) fasta_error();
  
  while(line != NULL) {
    if(strncmp(line,">",1) == 0) {
      n++;
      seqs = (char**)realloc(seqs, (n+2) * sizeof(char*));  
      names = (char**)realloc(names, (n+2) * sizeof(char*));
      seqs[n+1] = NULL;
      names[n+1] = NULL;
      
      if(strncmp(line,"> ",2) == 0) {
	names[n] = (char*)space((strlen(line)+1)*sizeof(char));
	sscanf(line, "%*s %s", names[n]);
	strcat(names[n], "\0");
      }
      else {
	tmp_name = (char*)space((strlen(line)+1)*sizeof(char));
	names[n] = (char*)space((strlen(line)+1)*sizeof(char));
	sscanf(line, "%s", tmp_name);
	for(i=1; i<strlen(tmp_name); i++){
	  names[n][i-1] = tmp_name[i];
	}
	strcat(names[n], "\0");
	free(tmp_name);
      }
    }
    else {
      if((line[0]) && (line[0] != ' ')){
	seqs_fragment = (char*)space((strlen(line)+1)*sizeof(char));
	sscanf(line, "%s", seqs_fragment);
	strcat(seqs_fragment, "\0");
	
	if(seqs[n] == NULL) {
	  seqs[n] = (char*)space((strlen(seqs_fragment)+1)*sizeof(char));
	  strcpy(seqs[n], seqs_fragment);
	}
	else {
	  seqs[n]=(char*)realloc(seqs[n], (strlen(seqs_fragment)+strlen(seqs[n])+1)*sizeof(char));
	  strcat(seqs[n], seqs_fragment);
	  strcat(seqs[n], "\0");
	}
	free(seqs_fragment);
      }
      else {
	free(line);
	fprintf(stderr, "Alignment error: corrupted line after identifier '%s'.\n\n", names[n]);
	exit(12);
      }
    }
    free(line);
    line = get_line(fasta);
  }
 
  for(i=0; i<n; i++){
    for(b=1; b<n+1; b++){
      if(strlen(seqs[i]) != strlen(seqs[b])){
	fprintf(stderr, "This is not an intact alignment file. Aborting.\n");
	exit(20);
      }
    }
  }
  
  i=0;
  while(seqs[i] != NULL) {
    j=0;
    while(seqs[i][j] != '\0') {
      seqs[i][j] = toupper(seqs[i][j]);
      if(seqs[i][j] == 'T') seqs[i][j] = 'U';
      j++;
    }
    i++;
  }

  for(i=0; i< n; i++){
    for(j=i+1; j< (n+1); j++){
      if(!strcmp(names[i], names[j])) severe_error(99);
    }
  }
  
  return n+1; /* number of ALL sequences (incl. constraint) */
}

/*--------------------------------------------------------------------------*/

int read_align_only(void) {

	char *line = NULL, *tmp_name = NULL;
	int k=0, n=-1, i, j, ch;	
	FILE *a_handle = NULL, *out_handle = NULL;
  
  fprintf(stdout, "\nReading and preprocessing sequence and structure input...\n");
  
  names = (char**)space(1000*sizeof(char*));
  seqs = (char**)space(1000*sizeof(char*));  
  strucs = (char**)space(1000*sizeof(char*));  


    for(k=0; k<1000; k++){
         seqs[k] = NULL;
         names[k] = NULL;
		 strucs[k] = NULL;
       }
  
  a_handle = fopen(options.align_only_FN, "r");
  out_handle = fopen("tmp.tmp", "w");
	  if(a_handle == NULL) {
    file_error(options.align_only_FN);
  }
  while((ch = fgetc (a_handle)) != EOF)	if(ch != '\015' && ch != '\032')   putc(ch, out_handle); 
  fclose(a_handle);
  fclose(out_handle);
  a_handle = fopen("tmp.tmp", "r");
  
  line=get_line(a_handle);
    
  if(strncmp(line,">",1) != 0) severe_error(112);
  
  while(line != NULL) {
    if(strncmp(line,">",1) == 0) {
		n++;
		seqs = (char**)realloc(seqs, (n+2) * sizeof(char*));  
		names = (char**)realloc(names, (n+2) * sizeof(char*));
		strucs = (char**)realloc(strucs, (n+2) * sizeof(char*));
		seqs[n] = seqs[n+1] = NULL;                                  /* hier eventuell .. ACHTUNG!!!! */
		names[n] = names[n+1] = NULL;
		strucs[n] = strucs[n+1] = NULL;
	  
		if(strncmp(line,"> ",2) == 0) {
			names[n] = (char*)space((strlen(line)+1)*sizeof(char));    /* hier eventuell .. (line)-1 ... */
			sscanf(line, "%*s %s", names[n]);
			strcat(names[n], "\0");
		}
		else {
			tmp_name = (char*)space((strlen(line)+1)*sizeof(char));
			names[n] = (char*)space((strlen(line)+1)*sizeof(char));
			sscanf(line, "%s", tmp_name);
			for(i=1; i<strlen(tmp_name); i++){
				names[n][i-1] = tmp_name[i];
			}
			strcat(names[n], "\0");
			free(tmp_name);  tmp_name = NULL;
		}
		free(line);
		line=get_line(a_handle);
		seqs[n] = (char*)space((strlen(line)+1)*sizeof(char));
		sscanf(line, "%s", seqs[n]);
		strcat(seqs[n], "\0");
	
		free(line);
		line=get_line(a_handle);
		strucs[n] = (char*)space((strlen(line)+1)*sizeof(char));
		sscanf(line, "%s", strucs[n]);
		strcat(strucs[n], "\0");
	}
	else severe_error(130);
	free(line);   line=get_line(a_handle);
	free(line);   line=get_line(a_handle);
  }
   fclose(a_handle);

  i=0;
  while(seqs[i] != NULL) {
    j=0;
    while(seqs[i][j] != '\0') {
      seqs[i][j] = toupper(seqs[i][j]);
      if(seqs[i][j] == 'T') seqs[i][j] = 'U';
      j++;
    }
    i++;
  }

  for(i=0; i< n; i++){
    for(j=i+1; j< (n+1); j++){
      if(!strcmp(names[i], names[j])) severe_error(99);
    }
  }
  
  return n+1; /* number of sequences  */
}

/*--------------------------------------------------------------------------*/

void free_aln_arrays() {

  int i=0;
  
  while(names[i] != NULL) {
    free(names[i]);
    free(seqs[i]);
  }
  free(names);
  free(seqs);
}
