#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structs.h"

extern struct input_options options;
extern struct seqStruct *seqLst;
int overruling_scores[5];
extern int align_only;

void parseArgs(int argc, char **argv);
void fasta_error(void);
void usage(void);
void file_error(char * file);
void version(void);

/* --------------------------------------- */

void parseArgs(int argc, char **argv) {

  int i, noI = 1, noC = 1;
  
  for(i=0; i<5; i++) overruling_scores[i] = 0;
  if(argc==1) usage();

  for(i=1; i < argc; i++) {
    if(argv[i][0] == '-') {
      switch(argv[i][1]) {
      case 'h':
	usage();
	break;
	case 'a':
	align_only = 1;
	
	
	
	if(argc == (i+1)){
	  fprintf(stderr, "\nArgument missing after -a. Aborting.\n\n");
	  exit(88);
	}
	strcpy(options.align_only_FN, argv[++i]);
	strcat(options.align_only_FN, "\0");
	
	break;
	case 'm':
	sscanf(argv[++i], "%i", &overruling_scores[1]);
	sscanf(argv[++i], "%i", &overruling_scores[2]);
	sscanf(argv[++i], "%i", &overruling_scores[3]);
	sscanf(argv[++i], "%i", &overruling_scores[4]);
	overruling_scores[0] =1;
	break;
      case 'v':
	version();
	break;
      case 'c':
	if(align_only) {
		fprintf(stderr, "\nOption -a already set! No constraint folding avilable. Exiting.\n\n"); exit(109);
	}
	if(argc == (i+1)){
	  fprintf(stderr, "Argument missing after -c. Aborting.\n\n");
	  exit(88);
	}
	strcpy(options.constraint_filename, argv[++i]);
	strcat(options.constraint_filename, "\0");
	noC = 0;
	break;
      case 'i':
	  if(align_only) {
		fprintf(stderr, "\nOption -a already set! No constraint folding avilable. Exiting.\n\n"); exit(109);
	}
	if(argc == (i+1)){
	  fprintf(stderr, "Argument missing after -i. Aborting.\n\n");
	  exit(88);
	}
	strcpy(options.input_filename, argv[++i]);
	strcat(options.input_filename, "\0");
	noI = 0;
	break;
      case 'p':   options.plot = 0;
	/* if(argv[i][2] == "0")     */
	break;
      case 's':
	switch(argv[i][2]){
	case '1':
	if(align_only) {
		fprintf(stderr, "\nOption -a already set! Only option -s3 is avilable. Exiting.\n\n"); exit(110);
	}
	  if(argc == (i+1)){
	    fprintf(stderr, "Argument missing after -s1. Aborting.\n\n");
	    exit(88);
	  }
	  sscanf(argv[++i], "%f", &options.SR1);
	  break;
	case '2':
	if(align_only) {
		fprintf(stderr, "\nOption -a already set! Only option -s3 is avilable. Exiting.\n\n"); exit(110);
	}
	  if(argc == (i+1)){
	    fprintf(stderr, "Argument missing after -s2. Aborting\n\n");
	    exit(88);
	  }
	  sscanf(argv[++i], "%f", &options.SR2);
	  break;
	case '3':
	  if(argc == (i+1)){
	    fprintf(stderr, "Argument missing after -s3. Aborting.\n\n");
	    exit(88);
	  }
	  sscanf(argv[++i], "%f", &options.SR3);
	  break;
	default: printf("\nUnknown option '%c'. Type 'RNAsalsa -h' for a list of options.\n\n", argv[i][2]);
	  exit(90);
	}
	break;
      case 'X':
	  if(align_only) {
		fprintf(stderr, "Option -a already set! Option -X is not available anymore. Exiting.\n"); exit(109);
	}
	options.kurzschluss = 1;
	break;
      default:
	printf("\nUnknown option '%c'. Type 'RNAsalsa -h' for a list of options.\n\n", argv[i][1]);
	exit(90);
      }
    }
    else usage();
  }
  
  if((noI || noC) && !align_only) usage();
  
}
/* --------------------------------------- */

void usage(void) {
  printf("\nUsage:\tRNAsalsa [-h] [-v] [-X] [-p] [-s1] [-s2] [-s3] -c FILE -i FILE\n");
  printf("   or:\tRNAsalsa [-h] [-v] [-s3] -a FILE\n\n");
  printf("%-8s Necessary switch and input file containing the constraint.\n", "-c FILE");
  printf("%-12s 1st line name, 2nd line sequence, 3rd line structure.\n"," ");
  printf("%-8s Necessary switch and input alignment file.\n", "-i FILE");
  printf("%-12s Input alignment in FASTA format.\n"," ");
  printf("%-8s Switch OFF all PostScript output of structures (default is ON).\n", "-p");
  printf("%-8s Stringency setting for initial constraint weakening.\n", "-s1");
  printf("%-8s Stringency setting for merging of pairwise consensus structures.\n", "-s2");
  printf("%-8s Stringency setting for building the final consensus structure.\n", "-s3");
  printf("%-12s Stringency values lie between 0.0 (lowest) and 1.0 (highest).\n"," ");
  printf("%-8s All pairwise alignments from input, no internal sequence alignments.\n", "-X");
  printf("%-8s Re-read existing RNAsalsa folding output, do multiple alignment only.\n", "-a FILE");
  printf("%-12s Folding output file format must equal 'SALSA_fold_results.txt'.\n"," ");
  printf("%-8s Show this help message and exit.\n", "-h");
  printf("%-8s Show version information and exit.\n\n", "-v");
  exit(1);
}

/*--------------------------------------------------------------------------*/

void version(void) {
  printf("\nVersion:\tRNAsalsa 0.8.1\n\n");
  printf("Development and implementation by Roman R Stocsits (co-writer: J Hertel)\n");
  printf("Thermodynamic folding algorithms from the Vienna RNA package.\n");
  printf("Please send comments and bug reports to rnasalsa@gmail.com\n\n");
  exit(1); 
}

/*--------------------------------------------------------------------------*/

void file_error(char *file) {
  fprintf(stderr, "\nERROR: Failed reading input file '%s'. Aborting.\n\n", file);
  exit(0);
}

/*--------------------------------------------------------------------------*/

void fasta_error(void){

  fprintf(stderr, "Input alignment is not FASTA format. Aborting.\n\n");
  exit(22);
}
